import "Turbine.UI";
import "Gillothrin.Items.Config";

Slider = class(Turbine.UI.Control);
function Slider:Constructor()
  Turbine.UI.Control.Constructor(self);
  self.ItemColor = Turbine.UI.Color(245/255, 222/255, 147/255);
  self.DisabledColor = Turbine.UI.Color(162/255, 162/255, 162/255);
  self.sliderLeftArrow = "Gillothrin/Items/resources/slider_leftarrow.tga";
  self.sliderLeftArrowPressed = "Gillothrin/Items/resources/slider_leftarrow_pressed.tga";
  self.sliderRightArrow = "Gillothrin/Items/resources/slider_rightarrow.tga";
  self.sliderRightArrowPressed = "Gillothrin/Items/resources/slider_rightarrow_pressed.tga";
  self.sliderBackground = "Gillothrin/Items/resources/slider_background.tga";
  self.sliderWidget = "Gillothrin/Items/resources/slider_widget.tga";
  self.sliderWidgetDisabled = "Gillothrin/Items/resources/slider_widget_ghosted.tga";

  self.step = 1;
  self.min = 0;
  self.max = 0;
  self.value = 0;
  self.format = "%d";

  -- text label
  self.label = Turbine.UI.Label();
  self.label:SetParent(self);
  self.label:SetPosition(0, 0);
  self.label:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
  self.label:SetForeColor(self.ItemColor );
  self.label:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft);

  -- value label
  self.valueLabel = Turbine.UI.Label();
  self.valueLabel:SetParent(self);
  self.valueLabel:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
  self.valueLabel:SetForeColor(self.ItemColor );
  self.valueLabel:SetTextAlignment(Turbine.UI.ContentAlignment.TopRight);
  self.valueLabel:SetMouseVisible(false);

  -- left arrow
  self.leftArrow = Turbine.UI.Control();
  self.leftArrow:SetParent(self);
  self.leftArrow:SetBackground(self.sliderLeftArrow);
  self.leftArrow:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
  self.leftArrow:SetSize(16,16);
  self.leftArrow.MouseDown = function(sender, args)
    if (not self:IsEnabled()) then
      return;
    end

    if (args.Button == Turbine.UI.MouseButton.Left) then
      self.leftArrow:SetWantsUpdates(true);
      self.leftArrow:SetBackground(self.sliderLeftArrowPressed);
      self.leftArrow.tick = Turbine.Engine.GetGameTime();
      self.leftArrow.wait = true;
    end
  end
  self.leftArrow.MouseUp = function(sender, args)
    if (not self:IsEnabled()) then
      return;
    end

    if (args.Button == Turbine.UI.MouseButton.Left) then
      self.leftArrow:SetWantsUpdates(false);
      self.leftArrow:SetBackground(self.sliderLeftArrow);
      self:Decrement();
    end
  end
  self.leftArrow.Update = function(sender, args)
    local gameTime = Turbine.Engine.GetGameTime();
    if (self.leftArrow.wait) then
      if ((gameTime - self.leftArrow.tick) > .5) then
        self.leftArrow.wait = false;
      end
    else
      if ((gameTime - self.leftArrow.tick) > .05) then
        self:Decrement();
        self.leftArrow.tick = gameTime;
      end
    end
  end

  -- right arrow
  self.rightArrow = Turbine.UI.Control();
  self.rightArrow:SetParent(self);
  self.rightArrow:SetBackground(self.sliderRightArrow);
  self.rightArrow:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
  self.rightArrow:SetSize(16,16);
  self.rightArrow.MouseDown = function(sender, args)
    if (not self:IsEnabled()) then
      return;
    end

    if (args.Button == Turbine.UI.MouseButton.Left) then
      self.rightArrow:SetWantsUpdates(true);
      self.rightArrow:SetBackground(self.sliderRightArrowPressed);
      self.rightArrow.tick = Turbine.Engine.GetGameTime();
      self.rightArrow.wait = true;
    end
  end
  self.rightArrow.MouseUp = function(sender, args)
    if (not self:IsEnabled()) then
      return;
    end

    if (args.Button == Turbine.UI.MouseButton.Left) then
      self.rightArrow:SetWantsUpdates(false);
      self.rightArrow:SetBackground(self.sliderRightArrow);
      self:Increment();
    end
  end
  self.rightArrow.Update = function(sender, args)
    local gameTime = Turbine.Engine.GetGameTime();
    if (self.rightArrow.wait) then
      if ((gameTime - self.rightArrow.tick) > .5) then
        self.rightArrow.wait = false;
      end
    else
      if ((gameTime - self.rightArrow.tick) > .05) then
        self:Increment();
        self.rightArrow.tick = gameTime;
      end
    end
  end

  -- slider area
  self.sliderBox = Turbine.UI.Control();
  self.sliderBox:SetParent(self);
  self.sliderBox:SetBackground(self.sliderBackground);
  self.sliderBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
  self.sliderBox.MouseClick = function(sender, args)
    if (not self:IsEnabled()) then
      return;
    end

    if (args.Button == Turbine.UI.MouseButton.Left) then
      local width = self.sliderBox:GetWidth() - 16;
      local x = args.X - 8;
      if (x < 0) then
        x = 0;
      end
      if(x > width) then
        x = width;
      end
      self.slider:SetPosition(x, 0);
      self:UpdateValueFromPosition();
    end
  end

  -- slider widget
  self.slider = Turbine.UI.Control();
  self.slider:SetParent(self.sliderBox);
  self.slider:SetBackground(self.sliderWidget);
  self.slider:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
  self.slider:SetSize(16,16)
  self.slider.MouseDown = function(sender, args)
    if (not self:IsEnabled()) then
      return;
    end

    if (args.Button == Turbine.UI.MouseButton.Left) then
      self.slider.dragStartX = args.X;
      self.slider.dragging = true;
    end
  end
  self.slider.MouseUp = function(sender, args)
    if (not self:IsEnabled()) then
      return;
    end

    if (args.Button == Turbine.UI.MouseButton.Left) then
      self.slider.dragging = false;
    end
  end
  self.slider.MouseMove = function(sender, args)
    if (not self:IsEnabled()) then
      return;
    end

    if (self.slider.dragging) then
      local left, top = self.slider:GetPosition();
      local width = self.sliderBox:GetWidth() - 16;

      local x = left - self.slider.dragStartX + args.X;
      if (x < 0) then
        x = 0;
      end
      if(x > width) then
        x = width;
      end
      self.slider:SetPosition(x, 0);
      self:UpdateValueFromPosition();
    end
  end
end

function Slider:SetText(text)
  self.label:SetText(text);
end

function Slider:SetSize(width, height)
  Turbine.UI.Control.SetSize(self, width, height);
  self:Layout();
end

function Slider:SetEnabled(enabled)
  Turbine.UI.Control.SetEnabled(self, enabled);
  if (enabled) then
    self.label:SetForeColor(self.ItemColor );
    self.valueLabel:SetForeColor(self.ItemColor );
    self.slider:SetBackground(self.sliderWidget);
  else
    self.label:SetForeColor(self.DisabledColor);
    self.valueLabel:SetForeColor(self.DisabledColor);
    self.slider:SetBackground(self.sliderWidgetDisabled);
  end
end

function Slider:Layout()
  local width, height = self:GetSize();

  self.label:SetSize(width * .75, 20);
  self.valueLabel:SetSize(width * .25, 20);
  self.valueLabel:SetPosition(width - (width * .25), 0);

  self.sliderBox:SetSize(width - 56, 16);
  self.sliderBox:SetPosition(28, 21);

  self.leftArrow:SetPosition(12, 21);
  self.rightArrow:SetPosition(width - 28, 21);

  -- update the slider position from the value now that our size has changed
  self:UpdatePositionFromValue();
end

function Slider:UpdateValueFromPosition()
  local x, y = self.slider:GetPosition();
  local width = self.sliderBox:GetWidth() - 16;
  local ppv = width / ((self.max - self.min) / self.step);

  self.value = (math.floor(x / ppv) * self.step) + self.min;
  self.valueLabel:SetText(string.format(self.format, self.value));

  ExecuteCallback(self, "ValueChanged", {value=self.value});
end

function Slider:UpdatePositionFromValue()
  local width = self.sliderBox:GetWidth() - 16;
  local ppv = width / ((self.max - self.min) / self.step);

  local x = (self.value - self.min) * ppv / self.step;
  self.slider:SetPosition(x, 0);
end

function Slider:SetValue(value)
  self.value = value;
  self.valueLabel:SetText(string.format(self.format, self.value));
  self:UpdatePositionFromValue();

  ExecuteCallback(self, "ValueChanged", {value=self.value});
end

function Slider:GetValue()
  return self.value;
end

function Slider:SetStep(step)
  self.step = step;
end

function Slider:SetMin(min)
  self.min = min;
end

function Slider:SetMax(max)
  self.max = max;
end

function Slider:SetFormat(format)
  self.format = format;
end

function Slider:Increment()
  local value = self.value + self.step;
  if (value > self.max) then
    value = self.max;
  end
  self:SetValue(value);
end

function Slider:Decrement()
  local value = self.value - self.step;
  if (value < self.min) then
    value = self.min;
  end
  self:SetValue(value);
end
